package iwallet

import (
	"github.com/spf13/cobra"

	"github.com/iost-official/go-iost/rpc/pb"
)

// callCmd represents the call command that call a contract with given actions.
var callCmd = &cobra.Command{
	Use:   "call [ACTION]...",
	Short: "Call the method in contracts",
	Long: `Call the method in contracts
	Would accept arguments as call actions or load transaction request directly from given file (which could be generated by "save" command).
	An ACTION is a group of 3 arguments: contract name, function name, method parameters.
	The method parameters should be a string with format '["arg0","arg1",...]'.`,
	Example: `  iwallet call "token.iost" "transfer" '["iost","user0001","user0002","123.45",""]' --account test0
  iwallet call "token.iost" "transfer" '["iost","user0001","user0002","123.45",""]' --output tx.json`,
	Args: func(cmd *cobra.Command, args []string) error {
		if outputTxFile == "" {
			return checkAccount(cmd)
		}
		return nil
	},
	RunE: func(cmd *cobra.Command, args []string) error {
		var actions []*rpcpb.Action
		actions, err := actionsFromFlags(args)
		if err != nil {
			return err
		}
		tx, err := initTxFromActions(actions)
		if err != nil {
			return err
		}
		return saveOrSendTx(tx)
	},
}

func init() {
	rootCmd.AddCommand(callCmd)
}
